@@ -51,6 +51,8 @@ rxnormV  <- function(mean = 0, sd = 1, n = 1L, ncores = 1L) {
 #' @export
 rxnorm <- rxnormV
 
+
+
 #' Simulate random Poisson variable from threefry generator
 #'
 #' @inheritParams stats::rpois
@@ -89,7 +91,6 @@ rxpois <- function(lambda, n = 1L, ncores = 1L) {
   .Call(`_rxode2_rxpois_`, lambda, n, ncores)
 }
 
-
 #' Simulate student t variable from threefry generator
 #'
 #' @inheritParams stats::rt
@@ -1283,3 +1284,10 @@ rxRmvn <- function(n, mu = NULL, sigma, lower = -Inf, upper = Inf, ncores = 1, i
   }
   return(.ret)
 }
+
+#' @export
+rxUdfUi.rxpois <- function(fun) {
+  .fun <- fun
+  .fun[[1]] <- str2lang(paste0(".", deparse1(fun[[1]])))
+  eval(.fun)
+}
