@@ -18,6 +18,7 @@ static inline double _safe_log_(double a, rx_solve *rx) {
     return log(a);
   }
 }
+
 #define _safe_log(a) _safe_log_(a, _solveData)
 static inline double _div0_(double denom, rx_solve *rx) {
   if (rx->safeZero) {
@@ -218,9 +219,14 @@ static inline double Rx_pow_di_(double a, double b, rx_solve *rx) {
 #define _logitInv1(x) expit(x, 0.0, 1.0)
 #define _logitInv2(x, y) expit(x, y, 1.0)
 #define _podo0() (_solveData->subjects[_cSub].curDose)
+#define _podo00() (ISNA(_solveData->subjects[_cSub].curDose) ? 0 : _solveData->subjects[_cSub].curDose)
 #define _podo1(x) (_solveData->subjects[_cSub].curDoseS[x])
+#define _podo01(x) (ISNA(_solveData->subjects[_cSub].curDoseS[x]) ? 0 : _solveData->subjects[_cSub].curDoseS[x])
+
 #define _dose0() (_solveData->subjects[_cSub].curDose)
 #define _dose1(x) (_solveData->subjects[_cSub].curDoseS[x])
+#define _dose00() (ISNA(_solveData->subjects[_cSub].curDose) ? 0 : _solveData->subjects[_cSub].curDose)
+#define _dose01(x) (ISNA(_solveData->subjects[_cSub].curDoseS[x]) ? 0 : _solveData->subjects[_cSub].curDoseS[x])
 #define _tad0() (t-_solveData->subjects[_cSub].tlast)
 #define _tad1(x) (t-_solveData->subjects[_cSub].tlastS[x])
 #define _tad00() (ISNA(_solveData->subjects[_cSub].tlast)? 0 : (t- _solveData->subjects[_cSub].tlast))
