@@ -2,50 +2,55 @@
 #define USE_FC_LEN_T
 #define STRICT_R_HEADERS
 #include <RcppArmadillo.h>
+#include "../inst/include/rxode2.h"
 #include <R.h>
 #include <threefry.h>
 #include <libintl.h>
 #include "checkmate.h"
 #include <boost/algorithm/string/predicate.hpp>
-#include "../inst/include/rxode2.h"
+#include "../inst/include/rxode2parse.h"
 #include "../inst/include/rxode2_as.h"
-extern "C"{
-  typedef SEXP (*lotriMat_type) (SEXP, SEXP, SEXP);
-  lotriMat_type lotriMat;
-  typedef SEXP (*asLotriMat_type) (SEXP, SEXP, SEXP);
-  asLotriMat_type asLotriMat;
-  typedef SEXP (*lotriSep_type) (SEXP, SEXP, SEXP, SEXP, SEXP);
-  lotriSep_type lotriSep;
-  typedef SEXP (*lotriAllNames_type) (SEXP);
-  lotriAllNames_type lotriAllNames;
-  typedef SEXP (*lotriGetBounds_type) (SEXP, SEXP, SEXP);
-  lotriGetBounds_type lotriGetBounds;
-  typedef SEXP (*isLotri_type) (SEXP);
-  isLotri_type isLotri;
-  typedef SEXP (*lotriMaxNu_type) (SEXP);
-  lotriMaxNu_type lotriMaxNu;
+#include <lotri.h>
+
+extern "C" {
+  iniLotri;
 }
 
+List etTrans(List inData, const RObject &mv, bool addCmt,
+             bool dropUnits, bool allTimeVar,
+             bool keepDosingOnly, Nullable<LogicalVector> combineDvid,
+             CharacterVector keep,
+             bool addlKeepsCov,
+             bool addlDropSs,
+             bool ssAtDoseTime,
+             Nullable<List> iCovIn);
+SEXP nestingInfo_(SEXP omega, List data);
+
+List rxExpandNesting(const RObject& obj, List& nestingInfo,
+                     bool compile=false);
+
+List rxModelVars_(const RObject &obj);
+
+extern "C" int get_sexp_uniqueL( SEXP s );
+
+extern "C" SEXP _cbindOme(SEXP et_, SEXP mat_, SEXP n_);
+
+extern "C" SEXP _vecDF(SEXP cv, SEXP n_);
+SEXP convertId_(SEXP x);
+
+SEXP rxRmvnSEXP(SEXP nS, SEXP muS, SEXP sigmaS,
+                SEXP lowerS, SEXP upperS, SEXP ncoresS, SEXP isCholS,
+                SEXP keepNamesS,
+                SEXP aS, SEXP tolS, SEXP nlTolS, SEXP nlMaxiterS);
+
+void rxModelsAssign(std::string str, SEXP assign);
+
+LogicalVector rxSolveFree();
+
 bool gotLotriMat=false;
 
-static inline void setupLotri() {
-  if (!gotLotriMat) {
-    lotriMat = (lotriMat_type) R_GetCCallable("lotri","_lotriLstToMat");
-    asLotriMat = (asLotriMat_type) R_GetCCallable("lotri","_asLotriMat");
-    lotriSep = (lotriSep_type) R_GetCCallable("lotri","_lotriSep");
-    lotriAllNames = (lotriAllNames_type) R_GetCCallable("lotri","_lotriAllNames");
-    lotriGetBounds = (lotriGetBounds_type) R_GetCCallable("lotri", "_lotriGetBounds");
-    isLotri = (isLotri_type) R_GetCCallable("lotri", "_isLotri");
-    lotriMaxNu = (lotriMaxNu_type) R_GetCCallable("lotri", "_lotriMaxNu");
-    gotLotriMat=true;
-  }
-}
-
 using namespace Rcpp;
 using namespace arma;
-bool rxIs(const RObject &obj, std::string cls);
-
-LogicalVector rxSolveFree();
 
 arma::mat rwish5(double nu, int p){
   // GetRNGstate();
@@ -81,10 +86,16 @@ NumericMatrix cvPost0(double nu, NumericMatrix omega, bool omegaIsChol = false,
     if (S.is_zero()) {
       ret[0] = 0.0;
     } else {
-      if (omegaIsChol){
-	ret[0] = nu*omega[0]*omega[0]/(Rf_rgamma(nu/2.0,2.0));
-      } else {
-	ret[0] = nu*omega[0]/(Rf_rgamma(nu/2.0,2.0));
+      bool retIsGood = false;
+      while (!retIsGood) {
+        if (omegaIsChol){
+          ret[0] = nu*omega[0]*omega[0]/(Rf_rgamma(nu/2.0,2.0));
+        } else {
+          ret[0] = nu*omega[0]/(Rf_rgamma(nu/2.0,2.0));
+        }
+        if (R_FINITE(ret[0])) {
+          retIsGood = true;
+        }
       }
       if (returnChol) ret[0] = sqrt(ret[0]);
     }
@@ -110,14 +121,14 @@ NumericMatrix cvPost0(double nu, NumericMatrix omega, bool omegaIsChol = false,
 //' Scaled Inverse Chi Squared distribution
 //'
 //' @param n Number of random samples
-//' 
+//'
 //' @param nu degrees of freedom of inverse chi square
-//' 
-//' @param scale  Scale of inverse chi squared distribution 
+//'
+//' @param scale  Scale of inverse chi squared distribution
 //'         (default is 1).
-//' 
+//'
 //' @return a vector of inverse chi squared deviates.
-//' 
+//'
 //' @examples
 //' rinvchisq(3, 4, 1) ## Scale = 1, degrees of freedom = 4
 //' rinvchisq(2, 4, 2) ## Scale = 2, degrees of freedom = 4
@@ -153,16 +164,16 @@ void rgbeta(int d, double shape, double* out){
 //' One correlation sample from the LKJ distribution
 //'
 //' @param d The dimension of the correlation matrix
-//' 
+//'
 //' @param eta The scaling parameter of the LKJ distribution.
 //'   Must be > 1.  Also related to the degrees of freedom nu.
 //'   eta = (nu-1)/2.
-//' 
+//'
 //' @param cholesky boolean; If `TRUE` return the cholesky
 //'   decomposition.
 //'
 //' @return A correlation sample from the LKJ distribution
-//' 
+//'
 //' @author Matthew Fidler (translated to RcppArmadillo) and Emma Schwager
 //' @export
 //' @keywords internal
@@ -232,13 +243,13 @@ arma::mat rLKJcvLsd1(arma::vec logSd, arma::vec logSdSD, double eta = 1.0){
 //' random covariate to a correlation.
 //'
 //' @inheritParams rLKJ1
-//' 
+//'
 //' @param nu Degrees of freedom of the Wishart distribution
-//' 
+//'
 //' @inheritParams cvPost
 //'
 //' @return One correlation sample from the inverse wishart
-//' 
+//'
 //' @author Matthew Fidler
 //' @keywords internal
 //' @export
@@ -247,7 +258,7 @@ arma::mat invWR1d(int d, double nu, bool omegaIsChol = false){
   if (nu <= d - 1) stop(_("'nu' must be greater than 'd'-1"));
   arma::mat I(d,d,arma::fill::eye);
   arma::mat invW = as<arma::mat>(cvPost0(nu, wrap(I),
-					 omegaIsChol, false));
+                                         omegaIsChol, false));
   arma::mat Dinv = diagmat(1/sqrt(invW.diag()));
   return Dinv * invW * Dinv;
 }
@@ -261,8 +272,8 @@ arma::mat rinvWRcv1(arma::vec sd, double nu = 1.0){
 
 //[[Rcpp::export]]
 arma::mat rcvC1(arma::vec sdEst, double nu = 3.0,
-		int diagXformType = 1, int rType = 1,
-		bool returnChol = false){
+                int diagXformType = 1, int rType = 1,
+                bool returnChol = false){
   // the sdEst should come from the multivariate normal distribution
   // with the appropriate transformation.
   unsigned int d = sdEst.size();
@@ -349,7 +360,7 @@ double getDbl(SEXP in, const char *var){
 
 //[[Rcpp::export]]
 SEXP cvPost_(SEXP nuS, SEXP omegaS, SEXP nS, SEXP omegaIsCholS,
-	     SEXP returnCholS, SEXP typeS, SEXP diagXformTypeS) {
+             SEXP returnCholS, SEXP typeS, SEXP diagXformTypeS) {
   int diagXformType = 1;
   qassertS(nS, "X1[1,)", "n");
   qassertS(omegaIsCholS, "B1", "omegaIsChol");
@@ -392,10 +403,15 @@ SEXP cvPost_(SEXP nuS, SEXP omegaS, SEXP nS, SEXP omegaIsCholS,
         }
         return as<SEXP>(cvPost0(nu, om2, omegaIsChol, returnChol));
       }
-    } else if (isLotri(omegaS)) {
-      RObject omega = omegaS;
+    } else if (!Rf_isNull(omegaS) && isLotri(omegaS)) {
+      RObject omega = as<RObject>(omegaS);
       List omegaIn = as<List>(omega);
-      CharacterVector omegaInNames = Rf_getAttrib(omega, R_NamesSymbol);
+      RObject omegaInNamesRO = Rf_getAttrib(omega, R_NamesSymbol);
+      if (Rf_isNull(omegaInNamesRO)) {
+        print(omega);
+        stop(_("the 'lotri' omega matrix needs to be named (printed above)"));
+      }
+      CharacterVector omegaInNames = as<CharacterVector>(omegaInNamesRO);
       int nOmega = omegaIn.size();
       List omegaLst(nOmega);
       List lotriLst = as<List>(omega.attr("lotri"));
@@ -442,8 +458,9 @@ SEXP cvPost_(SEXP nuS, SEXP omegaS, SEXP nS, SEXP omegaIsCholS,
       if (omega.hasAttribute("format")) {
         format = omega.attr("format");
       }
-      setupLotri();
-      return as<SEXP>(lotriMat(as<SEXP>(omegaLst), format, as<SEXP>(startAt)));
+      return as<SEXP>(lotriLstToMat(as<SEXP>(omegaLst), format,
+                                    as<SEXP>(startAt),
+                                    as<SEXP>(CharacterVector::create("matrix", "array"))));
     }
   } else {
     if (type == 1){
@@ -484,20 +501,27 @@ SEXP cvPost_(SEXP nuS, SEXP omegaS, SEXP nS, SEXP omegaIsCholS,
         arma::mat om0 = as<arma::mat>(omega);
         om0 = om0.t();
         List ret(om0.n_cols);
-        if (n != 1) Rf_warningcall(R_NilValue, _("'n' is determined by the 'omega' argument which contains the simulated standard deviations"));
+        if (n != 1) Rf_warningcall(R_NilValue, "%s", _("'n' is determined by the 'omega' argument which contains the simulated standard deviations"));
         for (unsigned int i = 0; i < om0.n_cols; i++){
           arma::vec sd = om0.col(i);
           if (nu < 3){
             stop("'nu' must be >= 3");
           }
-          arma::mat reti = rcvC1(sd, nu, diagXformType, type-1, returnChol);
+          bool goodSolve = false;
+          arma::mat reti;
+          while (!goodSolve) {
+            reti = rcvC1(sd, nu, diagXformType, type-1, returnChol);
+            if (reti.is_finite()) {
+              goodSolve = true;
+            }
+          }
           RObject retc = wrap(reti);
           retc.attr("dimnames") = omega.attr("dimnames");
           ret[i] = retc;
         }
         return(as<SEXP>(ret));
       } else {
-	stop(_("when sampling from correlation priors to create covariance matrices, the input must be a matrix of standard deviations"));
+        stop(_("when sampling from correlation priors to create covariance matrices, the input must be a matrix of standard deviations"));
       }
     }
   }
@@ -505,20 +529,13 @@ SEXP cvPost_(SEXP nuS, SEXP omegaS, SEXP nS, SEXP omegaIsCholS,
   return R_NilValue;
 }
 
-extern "C" SEXP _vecDF(SEXP cv, SEXP n_);
-void rxModelsAssign(std::string str, SEXP assign);
-
-SEXP rxRmvnSEXP(SEXP nS, SEXP muS, SEXP sigmaS,
-		SEXP lowerS, SEXP upperS, SEXP ncoresS, SEXP isCholS,
-		SEXP keepNamesS,
-		SEXP aS, SEXP tolS, SEXP nlTolS, SEXP nlMaxiterS);
 
 extern "C" void setZeroMatrix(int which);
 
 //[[Rcpp::export]]
 SEXP expandTheta_(SEXP thetaS, SEXP thetaMatS,
-		  SEXP thetaLowerS, SEXP thetaUpperS,
-		  SEXP nStudS, SEXP nCoresRVS) {
+                  SEXP thetaLowerS, SEXP thetaUpperS,
+                  SEXP nStudS, SEXP nCoresRVS) {
   if (Rf_isNull(thetaS)) {
     if (!Rf_isNull(thetaMatS)){
       stop(_("'thetaMat' needs 'params' to be non-NULL"));
@@ -529,7 +546,7 @@ SEXP expandTheta_(SEXP thetaS, SEXP thetaMatS,
   if (Rf_isNull(thetaMatS)) {
     if (Rf_isMatrix(thetaS)) {
       return as<SEXP>(as<DataFrame>(thetaS));
-    } else if (rxIs(thetaS, "data.frame")) {
+    } else if (Rf_inherits(thetaS, "data.frame")) {
       return thetaS;
     } else {
       return _vecDF(thetaS, nStudS);
@@ -579,15 +596,15 @@ SEXP expandTheta_(SEXP thetaS, SEXP thetaMatS,
       bool found = false;
       std::string curS = as<std::string>(theta00n[j]);
       for (R_xlen_t i = thetaMat.nrow(); i--;){
-	if (curS == as<std::string>(thetaMatDimNames[i])) {
-	  theta0[i] = theta00[j];
-	  found = true;
-	  break;
-	}
+        if (curS == as<std::string>(thetaMatDimNames[i])) {
+          theta0[i] = theta00[j];
+          found = true;
+          break;
+        }
       }
       if (!found) {
-	theta1[k] = theta00[j];
-	theta1n[k++] = theta00n[j];
+        theta1[k] = theta00[j];
+        theta1n[k++] = theta00n[j];
       }
     }
     theta1.names() = theta1n;
@@ -598,17 +615,17 @@ SEXP expandTheta_(SEXP thetaS, SEXP thetaMatS,
   }
   // Theta and thetaMat are correct, assign ".theta"
   rxModelsAssign(".theta", thetaMatS);
-  
+
   qassertS(nCoresRVS, "X1[1,)", "nCoresRV");
   NumericMatrix retNM = rxRmvnSEXP(nStudS, as<SEXP>(theta), as<SEXP>(thetaMat),
-				 thetaLowerS, thetaUpperS, nCoresRVS,
-				 as<SEXP>(LogicalVector::create(false)), // isChol
-				 as<SEXP>(LogicalVector::create(true)), // keepNames
-				 as<SEXP>(NumericVector::create(0.4)), // a
-				 as<SEXP>(NumericVector::create(2.05)), // tol
-				 as<SEXP>(NumericVector::create(1e-10)), // nlTol
-				 as<SEXP>(IntegerVector::create(100))
-				 );
+                                   thetaLowerS, thetaUpperS, nCoresRVS,
+                                   as<SEXP>(LogicalVector::create(false)), // isChol
+                                   as<SEXP>(LogicalVector::create(true)), // keepNames
+                                   as<SEXP>(NumericVector::create(0.4)), // a
+                                   as<SEXP>(NumericVector::create(2.05)), // tol
+                                   as<SEXP>(NumericVector::create(1e-10)), // nlTol
+                                   as<SEXP>(IntegerVector::create(100))
+                                   );
   int nrow = retNM.nrow();
   List ret(retNM.ncol()+theta1.size());
   CharacterVector retN(retNM.ncol()+theta1.size());
@@ -621,48 +638,37 @@ SEXP expandTheta_(SEXP thetaS, SEXP thetaMatS,
   for (R_xlen_t i = retNM.ncol(); i--;){
     NumericVector cur(nrow);
     std::copy(retNM.begin()+nrow*i, retNM.begin()+nrow*(i+1),
-	      cur.begin());
+              cur.begin());
     ret[theta1.size()+i] = cur;
     retN[theta1.size()+i] = thetaMatDimNames[i];
   }
   ret.names() = retN;
   Rf_setAttrib(ret, R_RowNamesSymbol,
-	       wrap(IntegerVector::create(NA_INTEGER, -nrow)));
+               wrap(IntegerVector::create(NA_INTEGER, -nrow)));
   Rf_setAttrib(ret, R_ClassSymbol,
-	       wrap("data.frame"));
+               wrap("data.frame"));
   return as<SEXP>(ret);
 }
 
 Function getRxFn(std::string name);
-
-SEXP chin(SEXP x, SEXP table);
-
-SEXP nestingInfo_(SEXP omega, List data);
-
-List rxExpandNesting(const RObject& obj, List& nestingInfo,
-		     bool compile=false);
-
-List rxModelVars_(const RObject &obj);
-
-extern "C" SEXP _cbindOme(SEXP et_, SEXP mat_, SEXP n_);
-
 static inline int getMethodInt(std::string& methodStr, CharacterVector& allNames, SEXP et) {
   int methodInt=1;
   if (methodStr == "auto") {
     // FIXME don't use %in%/%chin% from R
+    Function chin = getRxFn(".chin");
     LogicalVector inL = as<LogicalVector>(chin(allNames, Rf_getAttrib(et, R_NamesSymbol)));
     bool allIn = true;
     for (int j = inL.size(); j--;){
       if (!inL[j]) {
-	allIn = false;
-	break;
+        allIn = false;
+        break;
       }
     }
     if (allIn) {
       if (allNames.size() > 9) {
-	methodInt=3;//"separation";
+        methodInt=3;//"separation";
       } else {
-	methodInt = 2; //"ijk";
+        methodInt = 2; //"ijk";
       }
     } else {
       methodInt = 1;//"invWishart";
@@ -679,17 +685,11 @@ static inline int getMethodInt(std::string& methodStr, CharacterVector& allNames
   return methodInt;
 }
 
-List etTrans(List inData, const RObject &obj, bool addCmt=false,
-	     bool dropUnits=false, bool allTimeVar=false,
-	     bool keepDosingOnly=false, Nullable<LogicalVector> combineDvid=R_NilValue,
-	     CharacterVector keep = CharacterVector(0));
-
 //[[Rcpp::export]]
 SEXP expandPars_(SEXP objectS, SEXP paramsS, SEXP eventsS, SEXP controlS) {
   // SEXP events = as<DataFrame>(events);
   qassertS(controlS, "l+", "control");
   List control = as<List>(controlS);
-  setupLotri();
   int pro = 0;
   SEXP nStudS = PROTECT(control[Rxc_nStud]); pro++;
   int nStud = as<int>(nStudS);
@@ -698,8 +698,8 @@ SEXP expandPars_(SEXP objectS, SEXP paramsS, SEXP eventsS, SEXP controlS) {
   rxModelsAssign(".nestObj", objectS);
   rxModelsAssign(".nestEvents", eventsS);
   List et = expandTheta_(paramsS, control[Rxc_thetaMat],
-			 control[Rxc_thetaLower], control[Rxc_thetaUpper],
-			 nStudS, control[Rxc_nCoresRV]);
+                         control[Rxc_thetaLower], control[Rxc_thetaUpper],
+                         nStudS, control[Rxc_nCoresRV]);
   SEXP omegaS = PROTECT(control[Rxc_omega]); pro++;
   SEXP omegaLotri = R_NilValue;
   if (qtest(omegaS, "M")) {
@@ -721,13 +721,13 @@ SEXP expandPars_(SEXP objectS, SEXP paramsS, SEXP eventsS, SEXP controlS) {
     }
     SEXP omegaPre = PROTECT(wrap(omega)); pro++;
     Rf_setAttrib(omegaPre, R_DimNamesSymbol, as<SEXP>(dimnames));
-    
+
     // Convert to a lotri matrix
     omegaLotri = PROTECT(asLotriMat(omegaPre,
-				    as<SEXP>(List::create(_["lower"] = control[Rxc_omegaLower],
-							  _["upper"] = control[Rxc_omegaUpper],
-							  _["nu"]    = control[Rxc_dfSub])),
-				    as<SEXP>(CharacterVector::create("id")))); pro++;
+                                    as<SEXP>(List::create(_["lower"] = control[Rxc_omegaLower],
+                                                          _["upper"] = control[Rxc_omegaUpper],
+                                                          _["nu"]    = control[Rxc_dfSub])),
+                                    as<SEXP>(CharacterVector::create("id")))); pro++;
   } else if (isLotri(omegaS)) {
     omegaLotri = omegaS;
   } else if (Rf_isNull(omegaS)){
@@ -742,7 +742,7 @@ SEXP expandPars_(SEXP objectS, SEXP paramsS, SEXP eventsS, SEXP controlS) {
   SEXP lotriAbove = R_NilValue,
     lotriBelow = R_NilValue;
   int nid = 1;
-  List ni;    
+  List ni;
   CharacterVector allNames;
   std::string methodStr;
   int methodInt = 1;
@@ -767,15 +767,15 @@ SEXP expandPars_(SEXP objectS, SEXP paramsS, SEXP eventsS, SEXP controlS) {
       ni = nestingInfo_(omegaLotri, eventsL);
       IntegerVector idIV =  as<IntegerVector>(ni["id"]);//length(levels(.ni$id))
       nid = Rf_length(Rf_getAttrib(idIV, R_LevelsSymbol));
-      if (nid <= 1){
+      if (nid <= 1) {
       } else if (nSub <= 1) {
-	// control[Rxc_nSub] = nid;
-	nSub = nid;
+        // control[Rxc_nSub] = nid;
+        nSub = nid;
       } else if ((nSub*nStud) % nid != 0) {
-	rxSolveFree();
-	UNPROTECT(pro);
-	stop(_("provided multi-subject data (n=%d) trying to simulate a different number of subjects (n=%d)"),
-	     nid, nSub*nStud);
+        rxSolveFree();
+        UNPROTECT(pro);
+        stop(_("provided multi-subject data (n=%d) trying to simulate a different number of subjects (n=%d)"),
+             nid, nSub*nStud);
       }
       RObject objectRO = as<RObject>(objectS);
       IntegerVector flags = as<IntegerVector>(mv[RxMv_flags]);
@@ -785,9 +785,9 @@ SEXP expandPars_(SEXP objectS, SEXP paramsS, SEXP eventsS, SEXP controlS) {
       aboveSEXP = PROTECT(as<SEXP>(ni["above"])); pro++;
       belowSEXP = PROTECT(as<SEXP>(ni["below"])); pro++;
       List lotriSepMat = as<List>(PROTECT(lotriSep(omegaLotri,aboveSEXP,
-						   belowSEXP,
-						   as<SEXP>(IntegerVector::create(cureta)),
-						   as<SEXP>(IntegerVector::create(curtheta))))); pro++;
+                                                   belowSEXP,
+                                                   as<SEXP>(IntegerVector::create(cureta)),
+                                                   as<SEXP>(IntegerVector::create(curtheta))))); pro++;
       lotriAbove = lotriSepMat["above"];
       lotriBelow = lotriSepMat["below"];
       events = ni["data"];
@@ -801,9 +801,13 @@ SEXP expandPars_(SEXP objectS, SEXP paramsS, SEXP eventsS, SEXP controlS) {
       belowSEXP = omegaS;
       lotriBelow = omegaLotri;
       events = PROTECT(etTrans(as<List>(eventsS), nestObj,
-			       (INTEGER(mv[RxMv_flags])[RxMvFlag_hasCmt] == 1),
-			       false, false, true, R_NilValue,
-			       control[Rxc_keepF])); pro++;
+                               (INTEGER(mv[RxMv_flags])[RxMvFlag_hasCmt] == 1),
+                               false, false, true, R_NilValue,
+                               control[Rxc_keepF],
+                               control[Rxc_addlKeepsCov],
+                               control[Rxc_addlDropSs],
+                               control[Rxc_ssAtDoseTime],
+                               control[Rxc_iCov])); pro++;
       rxModelsAssign(".nestEvents", events);
       RObject cls = Rf_getAttrib(events, R_ClassSymbol);
       List rxLst = cls.attr(".rxode2.lst");
@@ -811,12 +815,12 @@ SEXP expandPars_(SEXP objectS, SEXP paramsS, SEXP eventsS, SEXP controlS) {
       nid = rxLst[RxTrans_nid];
       if (nid <= 1) {
       } else if (nSub <= 1) {
-	nSub = nid;
+        nSub = nid;
       } else if ((nSub*nStud) % nid != 0) {
-	rxSolveFree();
-	UNPROTECT(pro);
-	stop(_("provided multi-subject data (n=%d) trying to simulate a different number of subjects (n=%d)"),
-	     nid, nSub*nStud);
+        rxSolveFree();
+        UNPROTECT(pro);
+        stop(_("provided multi-subject data (n=%d) trying to simulate a different number of subjects (n=%d)"),
+             nid, nSub*nStud);
       }
       rxModelsAssign(".nestEta",    R_NilValue);
       rxModelsAssign(".nestTheta",  R_NilValue);
@@ -825,62 +829,74 @@ SEXP expandPars_(SEXP objectS, SEXP paramsS, SEXP eventsS, SEXP controlS) {
     methodStr = as<std::string>(control[Rxc_omegaSeparation]);
     methodInt = getMethodInt(methodStr, allNames, et);
 
+    if (Rf_isNull(lotriAbove)) {
+      aboveSEXP = R_NilValue;
+      rxModelsAssign(".nestEvents", events);
+      rxModelsAssign(".nestEta",    R_NilValue);
+      rxModelsAssign(".nestTheta",  R_NilValue);
+    }
+
     if (!Rf_isNull(aboveSEXP)) {
       // Create an extra theta matrix list
       //
       // Note this is for between study variability and the
-      // method/specification comes from omega, so methodInt comes from omegaSeparation
+      // method/specification comes from omega, so methodInt comes from
+      // omegaSeparation
+
       SEXP thetaList = PROTECT(cvPost_(et, lotriAbove,
-			       nStudS,
-			       LogicalVector::create(false),
-			       LogicalVector::create(false),
-			       IntegerVector::create(methodInt),
-				       control[Rxc_omegaXform])); pro++;
+                                       nStudS,
+                                       LogicalVector::create(false),
+                                       LogicalVector::create(false),
+                                       IntegerVector::create(methodInt),
+                                       control[Rxc_omegaXform])); pro++;
       if (Rf_length(thetaList) >= 1 &&
-	  asDouble(lotriMaxNu(lotriAbove), "lotriMaxNu(lotriAbove)") > 1.0) {
-	rxModelsAssign(".thetaL", thetaList);
+          asDouble(lotriMaxNu(lotriAbove), "lotriMaxNu(lotriAbove)") > 1.0) {
+        rxModelsAssign(".thetaL", thetaList);
       } else {
-	rxModelsAssign(".thetaL", R_NilValue);
+        rxModelsAssign(".thetaL", R_NilValue);
       }
       List bounds = PROTECT(lotriGetBounds(lotriAbove, R_NilValue, R_NilValue)); pro++;
       NumericVector upper = bounds[0];
       NumericVector lower = bounds[1];
-      // With 
+      // With
       NumericMatrix aboveMat = rxRmvnSEXP(IntegerVector::create(1),
-					R_NilValue, thetaList,
-					upper, lower, // lower upper 
-					control[Rxc_nCoresRV],
-					LogicalVector::create(false), // isChol
-					LogicalVector::create(true), // keepNames
-					NumericVector::create(0.4), // a
-					NumericVector::create(2.05), // tol
-					NumericVector::create(1e-10), // nlTol
-					IntegerVector::create(100)); // nlMaxiter
+                                          R_NilValue, thetaList,
+                                          upper, lower, // lower upper
+                                          control[Rxc_nCoresRV],
+                                          LogicalVector::create(false), // isChol
+                                          LogicalVector::create(true), // keepNames
+                                          NumericVector::create(0.4), // a
+                                          NumericVector::create(2.05), // tol
+                                          NumericVector::create(1e-10), // nlTol
+                                          IntegerVector::create(100)); // nlMaxiter
       DataFrame newLst = as<DataFrame>(aboveMat);
       if (!Rf_isNull(et) && Rf_length(et) != 0){
-	CharacterVector etListNames = asCv(Rf_getAttrib(et, R_NamesSymbol), "names(et)");
-	int baseSize = et.size();
-	List etFinal(baseSize + newLst.size());
-	CharacterVector etFinalNames(baseSize + newLst.size());
-	CharacterVector newLstNames = newLst.names();
-	for (int j = baseSize; j--;){
-	  etFinalNames[j] = etListNames[j];
-	  etFinal[j] = et[j];
-	}
-	for (int j = newLst.size(); j--;) {
-	  etFinalNames[baseSize+j] = newLstNames[j];
-	  etFinal[baseSize+j] = newLst[j];
-	}
-	Rf_setAttrib(etFinal, R_NamesSymbol, etFinalNames);
-	Rf_setAttrib(etFinal, R_RowNamesSymbol,
-		     IntegerVector::create(NA_INTEGER, -nStud));
-	Rf_setAttrib(etFinal, R_ClassSymbol,
-		     CharacterVector::create("data.frame"));
-	et = etFinal;
+        CharacterVector etListNames = asCv(Rf_getAttrib(et, R_NamesSymbol), "names(et)");
+        int baseSize = et.size();
+        List etFinal(baseSize + newLst.size());
+        CharacterVector etFinalNames(baseSize + newLst.size());
+        CharacterVector newLstNames = newLst.names();
+        for (int j = baseSize; j--;){
+          etFinalNames[j] = etListNames[j];
+          etFinal[j] = et[j];
+        }
+        for (int j = newLst.size(); j--;) {
+          etFinalNames[baseSize+j] = newLstNames[j];
+          etFinal[baseSize+j] = newLst[j];
+        }
+        Rf_setAttrib(etFinal, R_NamesSymbol, etFinalNames);
+        Rf_setAttrib(etFinal, R_RowNamesSymbol,
+                     IntegerVector::create(NA_INTEGER, -nStud));
+        Rf_setAttrib(etFinal, R_ClassSymbol,
+                     CharacterVector::create("data.frame"));
+        et = etFinal;
       }
     } else {
       rxModelsAssign(".thetaL", R_NilValue);
     }
+    if (Rf_isNull(lotriBelow)) {
+      belowSEXP = R_NilValue;
+    }
     if (!Rf_isNull(belowSEXP)) {
       // below to sample matrix
       SEXP omegaList = PROTECT(cvPost_(et, // In case needed
@@ -900,15 +916,15 @@ SEXP expandPars_(SEXP objectS, SEXP paramsS, SEXP eventsS, SEXP controlS) {
       NumericVector upper = bounds[0];
       NumericVector lower = bounds[1];
       NumericMatrix belowMat = rxRmvnSEXP(IntegerVector::create(nSub),
-					R_NilValue, omegaList,
-					upper, lower, // lower upper 
-					control[Rxc_nCoresRV],
-					LogicalVector::create(false), // isChol
-					LogicalVector::create(true), // keepNames
-					NumericVector::create(0.4), // a
-					NumericVector::create(2.05), // tol
-					NumericVector::create(1e-10), // nlTol
-					IntegerVector::create(100)); // nlMaxiter
+                                          R_NilValue, omegaList,
+                                          upper, lower, // lower upper
+                                          control[Rxc_nCoresRV],
+                                          LogicalVector::create(false), // isChol
+                                          LogicalVector::create(true), // keepNames
+                                          NumericVector::create(0.4), // a
+                                          NumericVector::create(2.05), // tol
+                                          NumericVector::create(1e-10), // nlTol
+                                          IntegerVector::create(100)); // nlMaxiter
       et = _cbindOme(wrap(et), belowMat, IntegerVector::create(nSub));
     } else {
       rxModelsAssign(".omegaL", R_NilValue);
@@ -942,10 +958,10 @@ SEXP expandPars_(SEXP objectS, SEXP paramsS, SEXP eventsS, SEXP controlS) {
     SEXP sigmaPre = PROTECT(wrap(sigma)); pro++;
     Rf_setAttrib(sigmaPre, R_DimNamesSymbol, as<SEXP>(dimnames));
     sigmaLotri = PROTECT(asLotriMat(sigmaPre,
-				    as<SEXP>(List::create(_["lower"] = control[Rxc_sigmaLower],
-							  _["upper"] = control[Rxc_sigmaUpper],
-							  _["nu"]    = control[Rxc_dfObs])),
-				    as<SEXP>(CharacterVector::create("id")))); pro++;
+                                    as<SEXP>(List::create(_["lower"] = control[Rxc_sigmaLower],
+                                                          _["upper"] = control[Rxc_sigmaUpper],
+                                                          _["nu"]    = control[Rxc_dfObs])),
+                                    as<SEXP>(CharacterVector::create("id")))); pro++;
   } else if (isLotri(sigmaS)) {
     sigmaLotri = sigmaS;
   } else if (!Rf_isNull(sigmaS)){
@@ -958,25 +974,33 @@ SEXP expandPars_(SEXP objectS, SEXP paramsS, SEXP eventsS, SEXP controlS) {
     methodStr = as<std::string>(control[Rxc_sigmaSeparation]);
     methodInt = getMethodInt(methodStr, allNames, et);
     SEXP sigmaList = PROTECT(cvPost_(et, // In case needed
-				     sigmaLotri,
-				     nStudS,
-				     LogicalVector::create(false),
-				     LogicalVector::create(false),
-				     IntegerVector::create(methodInt),
-				     control[Rxc_omegaXform])); pro++;
+                                     sigmaLotri,
+                                     nStudS,
+                                     LogicalVector::create(false),
+                                     LogicalVector::create(false),
+                                     IntegerVector::create(methodInt),
+                                     control[Rxc_omegaXform])); pro++;
     // To get the right number of sigma observations to match the potential request
     // expand the events to the translated events
     if (Rf_isNull(events)) {
       events = PROTECT(etTrans(as<List>(eventsS), nestObj,
-			       (INTEGER(mv[RxMv_flags])[RxMvFlag_hasCmt] == 1),
-			       false, false, true, R_NilValue,
-			       control[Rxc_keepF])); pro++;
+                               (INTEGER(mv[RxMv_flags])[RxMvFlag_hasCmt] == 1),
+                               false, false, true, R_NilValue,
+                               control[Rxc_keepF],
+                               control[Rxc_addlKeepsCov],
+                               control[Rxc_addlDropSs],
+                               control[Rxc_ssAtDoseTime],
+                               control[Rxc_iCov])); pro++;
       rxModelsAssign(".nestEvents", events);
-    } else if (!rxIs(events, "rxEtTrans")){
+    } else if (!Rf_inherits(events, "rxEtTrans")){
       events = PROTECT(etTrans(as<List>(events), nestObj,
-			       (INTEGER(mv[RxMv_flags])[RxMvFlag_hasCmt] == 1),
-			       false, false, true, R_NilValue,
-			       control[Rxc_keepF])); pro++;
+                               (INTEGER(mv[RxMv_flags])[RxMvFlag_hasCmt] == 1),
+                               false, false, true, R_NilValue,
+                               control[Rxc_keepF],
+                               control[Rxc_addlKeepsCov],
+                               control[Rxc_addlDropSs],
+                               control[Rxc_ssAtDoseTime],
+                               control[Rxc_iCov])); pro++;
       rxModelsAssign(".nestEvents", events);
     }
     int nobs =  Rf_length(VECTOR_ELT(events, 0));
@@ -990,16 +1014,16 @@ SEXP expandPars_(SEXP objectS, SEXP paramsS, SEXP eventsS, SEXP controlS) {
     NumericVector upper = bounds[0];
     NumericVector lower = bounds[1];
     SEXP sigmaMat = rxRmvnSEXP(n2, R_NilValue, sigmaList,
-			       upper, lower, // lower upper 
-			       control[Rxc_nCoresRV],
-			       LogicalVector::create(false), // isChol
-			       LogicalVector::create(true), // keepNames
-			       NumericVector::create(0.4), // a
-			       NumericVector::create(2.05), // tol
-			       NumericVector::create(1e-10), // nlTol
-			       IntegerVector::create(100)); // nlMaxiter
+                               upper, lower, // lower upper
+                               control[Rxc_nCoresRV],
+                               LogicalVector::create(false), // isChol
+                               LogicalVector::create(true), // keepNames
+                               NumericVector::create(0.4), // a
+                               NumericVector::create(2.05), // tol
+                               NumericVector::create(1e-10), // nlTol
+                               IntegerVector::create(100)); // nlMaxiter
     if (Rf_length(sigmaList) >= 1 &&
-	asDouble(lotriMaxNu(sigmaLotri), "lotriMaxNu(sigmaLotri)") > 1.0) {
+        asDouble(lotriMaxNu(sigmaLotri), "lotriMaxNu(sigmaLotri)") > 1.0) {
       rxModelsAssign(".sigmaL", sigmaList);
     } else {
       rxModelsAssign(".sigmaL", R_NilValue);
@@ -1043,9 +1067,6 @@ SEXP expandPars_(SEXP objectS, SEXP paramsS, SEXP eventsS, SEXP controlS) {
   return et;
 }
 
-SEXP convertId_(SEXP x);
-
-int get_sexp_uniqueL( SEXP s );
 int factor2( IntegerVector col, IntegerVector id) {
   IntegerVector x1(id.size());
   for (int i = id.size(); i--;) {
@@ -1120,9 +1141,8 @@ SEXP nestingInfo_(SEXP omega, List data) {
   if (Rf_isNewList(omega)){
     lotriOmega = omega;
   } else if (Rf_isMatrix(omega)) {
-    setupLotri();
     lotriOmega = PROTECT(asLotriMat(omega, R_NilValue,
-				    wrap(CharacterVector::create(idName)))); pro++;
+                                    wrap(CharacterVector::create(idName)))); pro++;
   } else {
     rxSolveFree();
     stop(_("'omega' must be a list/lotri/matrix"));
@@ -1150,26 +1170,27 @@ SEXP nestingInfo_(SEXP omega, List data) {
     if (lvl != idChar) {
       int found=-1;
       for (int j = lName.size(); j--;){
-	if (STRING_ELT(lNameSEXP,j) == lvl){
-	  found = j;
-	  break;
-	}
+        if (STRING_ELT(lNameSEXP,j) == lvl){
+          found = j;
+          break;
+        }
       }
+      if (found == -1) stop("could not find '%s' in data", CHAR(lvl));
       s = nestingInfoSingle_(data[found], id);
       l1 = Rf_length(Rf_getAttrib(s, R_LevelsSymbol));
       dn = VECTOR_ELT(Rf_getAttrib(VECTOR_ELT(lotriOmega, i),
-				   R_DimNamesSymbol), 0);
+                                   R_DimNamesSymbol), 0);
       SEXP nuSEXP = Rf_getAttrib(s, NuSymbol);
       if (Rf_isNull(nuSEXP)) {
-	aboveVars0[aboveI] = dn;
-	above[aboveI] = l1;
-	aboveN[aboveI++] = lvl;
-	extraTheta += Rf_length(dn) * l1;
+        aboveVars0[aboveI] = dn;
+        above[aboveI] = l1;
+        aboveN[aboveI++] = lvl;
+        extraTheta += Rf_length(dn) * l1;
       } else {
-	belowVars0[belowI] = dn;
-	below[belowI] = l1;
-	belowN[belowI++] = lvl;
-	extraEta += Rf_length(dn) * l1;
+        belowVars0[belowI] = dn;
+        below[belowI] = l1;
+        belowN[belowI++] = lvl;
+        extraEta += Rf_length(dn) * l1;
       }
       data[found] = s;
     }
@@ -1196,13 +1217,100 @@ SEXP nestingInfo_(SEXP omega, List data) {
   aboveVars.names() = aboveFN;
   UNPROTECT(pro);
   return wrap(List::create(_["data"]=data,
-			   _["omega"]=lotriOmega,
-			   _["idName"]=CharacterVector::create(idName),
-			   _["id"]=id,
-			   _["above"]=aboveF,
-			   _["below"]=belowF,
-			   _["aboveVars"]=aboveVars,
-			   _["belowVars"]=belowVars,
-			   _["extraTheta"]=extraTheta,
-			   _["extraEta"]=extraEta));
+                           _["omega"]=lotriOmega,
+                           _["idName"]=CharacterVector::create(idName),
+                           _["id"]=id,
+                           _["above"]=aboveF,
+                           _["below"]=belowF,
+                           _["aboveVars"]=aboveVars,
+                           _["belowVars"]=belowVars,
+                           _["extraTheta"]=extraTheta,
+                           _["extraEta"]=extraEta));
+}
+
+//[[Rcpp::export]]
+RObject swapMatListWithCube_(RObject inO) {
+  Rcpp::List omegaList;
+  if (inO.hasAttribute("dimnames")) {
+    Rcpp::List dimNamesFinal(2);
+    Rcpp::List dimNamesCur = inO.attr("dimnames");
+    dimNamesFinal[0] = dimNamesCur[0];
+    dimNamesFinal[1] = dimNamesCur[1];
+    arma::cube omegaCube = as<arma::cube>(inO);
+    Rcpp::List omegaList(omegaCube.n_slices);
+    for (int i = 0; i < omegaList.size(); ++i) {
+      RObject cur = wrap(omegaCube.slice(i));
+      cur.attr("dimnames") = dimNamesFinal;
+      omegaList[i] = cur;
+    }
+    return wrap(omegaList);
+  } else {
+    Rcpp::List omegaList = as<Rcpp::List>(inO);
+    RObject ro0 = omegaList[0];
+    arma::mat n0 = as<arma::mat>(omegaList[0]);
+    Rcpp::List dimNamesFinal(3);
+    bool hasDim = false;
+    if (ro0.hasAttribute("dimnames")) {
+      Rcpp::List dimNamesCur = ro0.attr("dimnames");
+      if (dimNamesCur.size() == 2) {
+        dimNamesFinal[0] = dimNamesCur[0];
+        dimNamesFinal[1] = dimNamesCur[1];
+        dimNamesFinal[2] = R_NilValue;
+      }
+      hasDim = true;
+    }
+    arma::cube omegaCube(n0.n_rows, n0.n_cols, omegaList.size());
+    for (int i = 0; i < omegaList.size(); ++i) {
+      omegaCube.slice(i) = as<arma::mat>(omegaList[i]);
+    }
+    RObject cur = wrap(omegaCube);
+    if (hasDim) cur.attr("dimnames") = dimNamesFinal;
+    return cur;
+  }
+  return R_NilValue;
+}
+
+//[[Rcpp::export]]
+Rcpp::List omegaListRse(RObject omegaIn) {
+  arma::cube omegaCube;
+  Rcpp::List omegaList;
+  bool useList = false;
+  unsigned int ntot=0;
+  if (omegaIn.hasAttribute("dim")) {
+    omegaCube = as<arma::cube>(omegaIn);
+    ntot = omegaCube.n_slices;
+  } else {
+    omegaList = as<Rcpp::List>(omegaIn);
+    useList = true;
+    ntot = omegaList.size();
+  }
+  arma::mat oldM;
+  if (useList) {
+    oldM = as<arma::mat>(omegaList[0]);
+  } else {
+    oldM = omegaCube.slice(0);
+  }
+  arma::mat newM = oldM;
+  arma::mat oldS(oldM.n_rows, oldM.n_rows, arma::fill::zeros);
+  arma::mat newS = oldS;
+  int m = 1;
+  for (unsigned int i = 1; i < ntot; i++) {
+    m++;
+    arma::mat x;
+    if (useList) {
+      x = as<arma::mat>(omegaList[i]);
+    } else {
+      x = omegaCube.slice(i);
+    }
+    newM = oldM + (x-oldM)/m;
+    newS = oldS + (x-oldM)*(x-newM);
+    oldM = newM;
+    oldS = newS;
+  }
+  arma::mat var = newS/(m-1);
+  arma::mat sd = sqrt(var);
+  return List::create(_["mean"] = wrap(newM),
+                      _["var"] = wrap(var),
+                      _["sd"] = wrap(sd),
+                      _["rse"]= wrap(sd/newM));
 }
