@@ -264,10 +264,14 @@ RObject rxSymInvCholEnvCalculate(List obj, std::string what, Nullable<NumericVec
     if (what == "theta"){
       NumericVector par(theta);
       int ntheta = as<int>(rxSymInvCholEnvCalculate(obj, "ntheta", R_NilValue));
-      if (par.size() == ntheta){
+      if (par.size() == ntheta) {
         // Clear cache with the exception of
-        CharacterVector sym = e.ls(TRUE);
-	// Clear the cache
+        Function ls2("ls", R_BaseNamespace);
+        CharacterVector sym = ls2(_["envir"]=e,
+                                  _["all.names"]=true,
+                                  _["sorted"]=false);
+
+        // Clear the cache
         for (int i = 0; i < sym.size(); i++){
           if (sym[i] != "invobj" && sym[i] != "ntheta") {
             e.remove(as<std::string>(sym[i]));
